@@ -88,6 +88,7 @@ static void resolveInstantiatedClassTemplate(sipSpec *pt, argDef *type);
 static void setStringPoolOffsets(sipSpec *pt);
 static const char *templateString(const char *src, scopedNameDef *names,
         scopedNameDef *values);
+static mappedTypeDef *copyTemplateType(mappedTypeDef *mtd, argDef *ad);
 
 
 /*
@@ -514,8 +515,13 @@ static void checkAssignmentHelper(sipSpec *pt, classDef *cd)
         if (ct->cppsig == NULL || !isPublicCtor(ct))
             continue;
 
-        if (ct->cppsig->nrArgs == 0)
+        if (ct->cppsig->nrArgs == 0 || ct->cppsig->args[0].defval != NULL)
+        {
+            /*
+             * The ctor either has no arguments or all arguments have defaults.
+             */
             pub_def_ctor = TRUE;
+        }
         else if (ct->cppsig->nrArgs == 1)
         {
             argDef *ad = &ct->cppsig->args[0];
@@ -1862,9 +1868,16 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     /* Handle the Python signature. */
     resolvePySigTypes(pt, mod, c_scope, od, &od->pysig, isSignal(od));
 
-    /* These slots must return int. */
     res = &od->pysig.result;
 
+    /* These slots must return SIP_SSIZE_T (or int - deprecated). */
+    if (isSSizeReturnSlot(od->common))
+        if ((res->atype != ssize_type && res->atype != int_type) || res->nrderefs != 0 ||
+            isReference(res) || isConstArg(res))
+            fatal("%s slots must return SIP_SSIZE_T\n",
+                    od->common->pyname->text);
+
+    /* These slots must return int. */
     if (isIntReturnSlot(od->common))
         if (res->atype != int_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
@@ -1944,7 +1957,7 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
                     fatal("::");
                 }
 
-                fatal("%s() unsupported signal argument type\n");
+                fatal("%s() unsupported signal argument type\n", od->cppname);
             }
         }
         else if (!supportedType(scope,od,ad,TRUE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL || (isVirtual(od) && od -> virthandler -> virtcode == NULL)))
@@ -2019,6 +2032,7 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     case long_type:
     case ulonglong_type:
     case longlong_type:
+    case ssize_type:
     case pyobject_type:
     case pytuple_type:
     case pylist_type:
@@ -2148,6 +2162,7 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case long_type:
     case ulonglong_type:
     case longlong_type:
+    case ssize_type:
     case pyobject_type:
     case pytuple_type:
     case pylist_type:
@@ -2157,6 +2172,12 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case pytype_type:
         if (isReference(ad))
         {
+            if (isConstArg(ad))
+            {
+                ensureInput(cd, od, ad);
+                return TRUE;
+            }
+
             if (ad -> nrderefs == 0 && outputs)
             {
                 defaultOutput(ad);
@@ -2423,7 +2444,7 @@ int sameSignature(signatureDef *sd1,signatureDef *sd2,int strict)
             (t) == latin1_string_type || (t) == utf8_string_type)
 #define pyAsFloat(t)    ((t) == cfloat_type || (t) == float_type || \
             (t) == cdouble_type || (t) == double_type)
-#define pyAsInt(t)  ((t) == bool_type || \
+#define pyAsInt(t)  ((t) == bool_type || (t) == ssize_type || \
             (t) == short_type || (t) == ushort_type || \
             (t) == cint_type || (t) == int_type || (t) == uint_type)
 #define pyAsLong(t) ((t) == long_type || (t) == longlong_type)
@@ -2899,6 +2920,8 @@ static mappedTypeDef *instantiateMappedTypeTemplate(sipSpec *pt, moduleDef *mod,
     if (type_values != NULL)
         freeScopedName(type_values);
 
+    mtd = copyTemplateType(mtd, type);
+
     return mtd;
 }
 
@@ -3052,6 +3075,8 @@ static void searchMappedTypes(sipSpec *pt, moduleDef *context,
                     continue;
             }
 
+            mtd = copyTemplateType(mtd, ad);
+
             /* Copy the type. */
             ad->atype = mapped_type;
             ad->u.mtd = mtd;
@@ -3068,6 +3093,50 @@ static void searchMappedTypes(sipSpec *pt, moduleDef *context,
 }
 
 
+/*
+ * If a mapped type is based on a template then create a copy that keeps the
+ * original types of the template arguments.
+ */
+static mappedTypeDef *copyTemplateType(mappedTypeDef *mtd, argDef *ad)
+{
+    int a;
+    signatureDef *src, *dst;
+    mappedTypeDef *mtd_copy;
+
+    /* There is no need to do anything for non-template types. */
+    if (mtd->type.atype != template_type)
+        return mtd;
+
+    /* Retain the original types if there are any. */
+    mtd_copy = mtd;
+    src = &ad->u.td->types;
+
+    for (a = 0; a < src->nrArgs; ++a)
+    {
+        typedefDef *tdd = src->args[a].original_type;
+
+        if (tdd != NULL)
+        {
+            /*
+             * Create the copy now that we know it is needed and if it hasn't
+             * already been done.
+             */
+            if (mtd_copy == mtd)
+            {
+                mtd_copy = sipMalloc(sizeof (mappedTypeDef));
+                *mtd_copy = *mtd;
+
+                dst = &mtd_copy->type.u.td->types;
+            }
+
+            dst->args[a].original_type = tdd;
+        }
+    }
+
+    return mtd_copy;
+}
+
+
 /*
  * Search the typedefs for a name and return the type.
  */
