@@ -54,8 +54,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040a04
-#define SIP_VERSION_STR     "snapshot-4.10.4-f5574a061fd0"
+#define SIP_VERSION         0x040b01
+#define SIP_VERSION_STR     "4.11.1-snapshot-13632c7f0f2c"
 
 
 /*
@@ -68,6 +68,9 @@ extern "C" {
  *
  * History:
  *
+ * 8.0  Changed the size of the sipSimpleWrapper structure.
+ *      Added sip_api_get_address().
+ *
  * 7.1  Added the 'H' format character to sip_api_parse_result().
  *      Deprecated the 'D' format character of sip_api_parse_result().
  *
@@ -168,8 +171,8 @@ extern "C" {
  *
  * 0.0  Original version.
  */
-#define SIP_API_MAJOR_NR    7
-#define SIP_API_MINOR_NR    1
+#define SIP_API_MAJOR_NR    8
+#define SIP_API_MINOR_NR    0
 
 
 /* Some compatibility stuff to help with handwritten code for SIP v3. */
@@ -208,6 +211,10 @@ extern "C" {
 #define SIPBytes_AS_STRING  PyBytes_AS_STRING
 #define SIPBytes_GET_SIZE   PyBytes_GET_SIZE
 
+#if PY_MINOR_VERSION >= 1
+#define SIP_USE_PYCAPSULE
+#endif
+
 #else
 
 #define SIPLong_FromLong    PyInt_FromLong
@@ -219,6 +226,10 @@ extern "C" {
 #define SIPBytes_AS_STRING  PyString_AS_STRING
 #define SIPBytes_GET_SIZE   PyString_GET_SIZE
 
+#if PY_MINOR_VERSION >= 7
+#define SIP_USE_PYCAPSULE
+#endif
+
 #endif
 
 #if !defined(Py_REFCNT)
@@ -261,6 +272,48 @@ typedef int sip_gilstate_t;
 #endif
 
 
+/*
+ * Some convenient function pointers.
+ */
+
+/*
+ * The operation an access function is being asked to perform.
+ */
+typedef enum
+{
+    UnguardedPointer,   /* Return the unguarded pointer. */
+    GuardedPointer,     /* Return the guarded pointer, ie. 0 if it has gone. */
+    ReleaseGuard        /* Release the guard, if any. */
+} AccessFuncOp;
+
+typedef void *(*sipInitFunc)(struct _sipSimpleWrapper *, PyObject *,
+        PyObject *, PyObject **, PyObject **, PyObject **);
+typedef void *(*sipAccessFunc)(struct _sipSimpleWrapper *, AccessFuncOp);
+typedef int (*sipTraverseFunc)(void *, visitproc, void *);
+typedef int (*sipClearFunc)(void *);
+#if PY_MAJOR_VERSION >= 3
+typedef int (*sipGetBufferFunc)(PyObject *, void *, Py_buffer *, int);
+typedef void (*sipReleaseBufferFunc)(PyObject *, void *, Py_buffer *);
+#else
+typedef SIP_SSIZE_T (*sipBufferFunc)(PyObject *, void *, SIP_SSIZE_T, void **);
+typedef SIP_SSIZE_T (*sipSegCountFunc)(PyObject *, void *, SIP_SSIZE_T *);
+#endif
+typedef void (*sipDeallocFunc)(struct _sipSimpleWrapper *);
+typedef void *(*sipCastFunc)(void *, const struct _sipTypeDef *);
+typedef const struct _sipTypeDef *(*sipSubClassConvertFunc)(void **);
+typedef int (*sipConvertToFunc)(PyObject *, void **, int *, PyObject *);
+typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
+typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
+typedef void (*sipAssignFunc)(void *, SIP_SSIZE_T, const void *);
+typedef void *(*sipArrayFunc)(SIP_SSIZE_T);
+typedef void *(*sipCopyFunc)(const void *, SIP_SSIZE_T);
+typedef void (*sipReleaseFunc)(void *, int);
+typedef PyObject *(*sipPickleFunc)(void *);
+typedef int (*sipAttrGetterFunc)(const struct _sipTypeDef *, PyObject *);
+typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *);
+typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
+
+
 /*
  * The meta-type of a wrapper type.
  */
@@ -288,13 +341,14 @@ typedef struct _sipWrapperType {
 typedef struct _sipSimpleWrapper {
     PyObject_HEAD
 
-    union {
-        /* C/C++ object pointer. */
-        void *cppPtr;
+    /*
+     * The data, initially a pointer to the C/C++ object, as interpreted by the
+     * access function.
+     */
+    void *data;
 
-        /* Access function. */
-        void *(*afPtr)();
-    } u;
+    /* The optional access function. */
+    sipAccessFunc access_func;
 
     /* Object flags. */
     int flags;
@@ -350,36 +404,6 @@ typedef struct _sipEnumTypeObject {
 } sipEnumTypeObject;
 
 
-/*
- * Some convenient function pointers.
- */
-typedef void *(*sipInitFunc)(sipSimpleWrapper *, PyObject *, PyObject *,
-        PyObject **, PyObject **, PyObject **);
-typedef int (*sipTraverseFunc)(void *, visitproc, void *);
-typedef int (*sipClearFunc)(void *);
-#if PY_MAJOR_VERSION >= 3
-typedef int (*sipGetBufferFunc)(PyObject *, void *, Py_buffer *, int);
-typedef void (*sipReleaseBufferFunc)(PyObject *, void *, Py_buffer *);
-#else
-typedef SIP_SSIZE_T (*sipBufferFunc)(PyObject *, void *, SIP_SSIZE_T, void **);
-typedef SIP_SSIZE_T (*sipSegCountFunc)(PyObject *, void *, SIP_SSIZE_T *);
-#endif
-typedef void (*sipDeallocFunc)(sipSimpleWrapper *);
-typedef void *(*sipCastFunc)(void *, const struct _sipTypeDef *);
-typedef const struct _sipTypeDef *(*sipSubClassConvertFunc)(void **);
-typedef int (*sipConvertToFunc)(PyObject *, void **, int *, PyObject *);
-typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
-typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
-typedef void (*sipAssignFunc)(void *, SIP_SSIZE_T, const void *);
-typedef void *(*sipArrayFunc)(SIP_SSIZE_T);
-typedef void *(*sipCopyFunc)(const void *, SIP_SSIZE_T);
-typedef void (*sipReleaseFunc)(void *, int);
-typedef PyObject *(*sipPickleFunc)(void *);
-typedef int (*sipAttrGetterFunc)(const struct _sipTypeDef *, PyObject *);
-typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *);
-typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
-
-
 /*
  * The information describing an encoded type ID.
  */
@@ -1359,6 +1383,7 @@ typedef struct _sipAPIDef {
             PyObject *sipKwdArgs, const char **kwdlist, PyObject **unused,
             const char *fmt, ...);
     void (*api_add_exception)(sipErrorState es, PyObject **parseErrp);
+    void *(*api_get_address)(struct _sipSimpleWrapper *w);
 } sipAPIDef;
 
 
